/*
  NIRHebrewService.m

  HebrewService - system services for Hebrew users.
 
  Copyright (C) 2003-2006 Nir Soffer
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details:
  http://www.gnu.org/licenses/gpl.html
*/

#import "NIRHebrewService.h"
#import "NSString-NirAdditions.h"
#import "NSString-FriBidiAdditions.h"
#import "NSPasteboard-NirAdditions.h"


@implementation NIRHebrewService

- (void)convertFromHebrewWindows:(NSPasteboard *)pboard
                        userData:(NSString *)data
                           error:(NSString **)error; {
    /* Convert the selection from ISO Latin 1 */

    // Get string from the pasteboard
    NSString *selection = [self stringFromPasteboard:pboard];
    if (selection == nil) {
        *error = @"Could not get selection from the pasteboard";
        return;
    }

    /*
     The string we get is Hebrew string using cp1255 encoding, but it was 
     decoded as iso-8859-1 string. To fix it, we first encode it back to 
     iso-8859-1, then decode it again as cp1255.
     */
    
    // Try to encode using iso-8859-1, replacing what can not be converted
    NSData *tmp = [selection dataUsingEncoding:NSISOLatin1StringEncoding
                          allowLossyConversion:YES];
    if (tmp == nil) {
        *error = @"Could not encode selection as iso-8859-1 string";
        return;
    }
    
    // Try to decode using cp1255
    UInt32 encoding = 
        CFStringConvertEncodingToNSStringEncoding(kCFStringEncodingWindowsHebrew);
    NSString *result = [[NSString alloc] initWithData:tmp encoding:encoding];
    if (result == nil) {
        *error = @"Could not decode selection as Hebrew (Windows)";
        return;
    }
    [result autorelease];
    
    // Return string
    if (! [pboard putObject:result forType:NSStringPboardType]) {
        *error = @"Could not put converted string on the pasteboard";
    }
}

- (void)reverseSelection:(NSPasteboard *)pboard
                userData:(NSString *)data
                   error:(NSString **)error; {
    /* Reverse the selection using Bidi algorithm */

    // Get string from the pasteboard
    NSString *selection = [self stringFromPasteboard:pboard];
    if (selection == nil) {
        *error = @"Could not get selection from the pasteboard";
        return;
    }

    // Reverse
    NSString *result = [selection stringByOrderingLinesVisuallyUsingDirection:
        FRIBIDI_TYPE_RTL];
    if (result == nil) {
        *error = @"Could not reverse selection";
        return;
    }

    // Return string
    if (! [pboard putObject:result forType:NSStringPboardType]) {
        *error = @"Could not put reversed string on the pasteboard";
    }
}

- (void)unwrapSelection:(NSPasteboard *)pboard
               userData:(NSString *)data
                  error:(NSString **)error; {
    /* unwrap lines in the selection */
    
    // Get string from the pasteboard
    NSString *selection = [self stringFromPasteboard:pboard];
    if (selection == nil) {
        *error = @"Could not get selection from the pasteboard";
        return;
    }
    
    // convert
    NSString *result = [selection stringByUnwrappingLines];
    if (result == nil) {
        *error = @"Could not reverse selection";
        return;
    }
    
    // Return string
    if (! [pboard putObject:result forType:NSStringPboardType]) {
        *error = @"Could not put reversed string on the pasteboard";
    }
}

- (void)escapeURL:(NSPasteboard *)pboard
         userData:(NSString *)data
            error:(NSString **)error; {
    /* Escape URL using utf-8 encoding */
    
    // Get string from the pasteboard
    NSString *selection = [self stringFromPasteboard:pboard];
    if (selection == nil) {
        *error = @"Could not get selection from the pasteboard";
        return;
    }
    
    // Escape
    NSString *result = [selection stringByAddingPercentEscapesUsingEncoding:
        NSUTF8StringEncoding];
    if (result == nil) {
        *error = @"Could not escape URL";
        return;
    }
    
    // Return string
    if (! [pboard putObject:result forType:NSStringPboardType]) {
        *error = @"Could not put escaped string on the pasteboard";
    }
}

- (NSString *)stringFromPasteboard:(NSPasteboard *)pboard; {
    /* get string from pasteboard */

    NSArray *types = [[NSArray alloc] initWithObjects:NSStringPboardType, nil];
    NSString *type = [pboard availableTypeFromArray:types];
    [types release];
    if (type != nil) {
        return [pboard stringForType:type];
    }
    return nil;
}

@end
